part of edit_screen;

class EditorController {
  final List<EditorItem> editorItems;
  final Map<String, Object> dataEntity = {};
  final String onSaveProcedure;
  String _messageInvalidDialog;
  bool _isDirtyData = false;
  RxBool _isConfirm = false.obs;
  RxBool _isWaiting = false.obs;
  RxBool _succeed = RxBool(false);
  RxBool _isShowInValidDialog = false.obs;
  VoidCallback onSuccess;
  static Map<String,Object> mapDataReturn;

  EditorController(
    Map<String, Object> dataEntity,
    this.onSaveProcedure,
    this.editorItems,
  ) {
    _addDefaultValues();
    initalValue(dataEntity);
  }

  void initalValue(Map<String, Object> initialMapValue) {
    if (initialMapValue != null) {
      dataEntity.addEntries(initialMapValue.entries);
    }
  }

  void onValueChanged(String fieldName, Object value) {
    Object _value = value;
    final editorItem = _getEditorItemByFieldName(fieldName);
    if (editorItem?.type == EEditorType.int && (value is String)) {
      _value = int.tryParse(value);
    } else if (editorItem?.type == EEditorType.double && (value is String)) {
      _value = double.tryParse(value);
    } else if (editorItem?.type == EEditorType.money && (value is String)) {
      _value = double.tryParse(value.replaceAll(RegExp(','), ''));
    }

    dataEntity.update(
      fieldName,
      (_) => _value,
      ifAbsent: () => _value,
    );
    _isDirtyData = true;
  }

  void confirmAndSave({VoidCallback onSuccess}) {
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    _isConfirm.value = true;
  }

  void showInvalidDialog({String message}) {
    _messageInvalidDialog = message;
    _isShowInValidDialog.value = true;
  }

  Future<Map<String,Object>> save({VoidCallback onSuccess}) async {
    Map<String,Object> dataReturn={};
    if (onSuccess != null) {
      this.onSuccess = onSuccess;
    }
    if (_isDirtyData) {
      final Map<String, Object> jParam = dataEntity.map((key, value) {
        return MapEntry('@$key', value);
      });

      jParam.removeWhere(
          (key, value) => _checkIsImagesKey(key.substring(1, key.length)));

      try {
        _isWaiting.value = true;
        print('EditorWidget: Submitting...');

        final response =
            await SQLService.execute(onSaveProcedure, params: jParam);
        print('result: $response');
        dataReturn=response;
        if (response['Exception']?.isNotEmpty ?? false) {
          _succeed.value = false;
          print(
              'Editor Save Error On Server: ${response['Exception']?.first['Message']}');
        } else {
          await _saveImages(response);
          _succeed.value = true;
        }
      } catch (e, stackTrace) {
        print('Editor Save Error: $e');
        print(stackTrace);
      } finally {
        print('finally Save Error:');
      }
      mapDataReturn = dataReturn;
      return dataReturn;
    } else {
      print('data have already save');
      mapDataReturn = dataReturn;
      return dataReturn;
    }
  }

// #region TextField
  String getHintText(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem?.type == EEditorType.money) {
      return Message.getMessage('EditorWidget_nhap_so_tien');
    }
    if (editorItem.title != null) {
      return Message.getMessage('EditorWidget_nhap') +
          ' ' +
          _removeLastStarSymbolAndLowCase(editorItem.title);
    }
    return Message.getMessage('EditorWidget_nhap_thong_tin');
  }

  String getText(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null || dataEntity[fieldName] == null) return '';

    if (editorItem.type == EEditorType.money) {
      return PriceInputFormatter.format(
          (dataEntity[fieldName] as double).toInt().toString());
    }

    return dataEntity[fieldName].toString();
  }

  String getValidator(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null || (!editorItem.isRequired && _isEmpty(fieldName))) {
      return null;
    }

    if (editorItem.validator != null && dataEntity[fieldName] != null) {
      return editorItem.validator(dataEntity[fieldName]);
    }

    if (_isEmpty(fieldName)) {
      return _getEmptyMessageValidator(editorItem.title);
    }

    switch (editorItem.type) {
      case EEditorType.image:
        return _validateImage(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(editorItem.title);
      case EEditorType.string:
        return _validateString(dataEntity[fieldName])
            ? null
            : _getEmptyMessageValidator(
                editorItem.title,
              );
      case EEditorType.phone:
        return Methods.validPhoneNumber(dataEntity[fieldName] ?? '')
            ? null
            : _getInvalidMessageValidator(
                Message.getMessage('EditorWidget_so_dien_thoai'));
      case EEditorType.email:
        return (dataEntity[fieldName] as String).isEmail
            ? null
            : _getInvalidMessageValidator(
                Message.getMessage('EditorWidget_email'));
      case EEditorType.int:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem.title);
      case EEditorType.double:
        return dataEntity[fieldName] != null
            ? null
            : _getEmptyMessageValidator(editorItem.title);
      case EEditorType.money:
        return _validateMoney(dataEntity[fieldName])
            ? null
            : editorItem.invalidMessage ??
                Message.getMessage('EditorWidget_price_validator');
        break;
    }
    return null;
  }

  TextInputType getTextInputType(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null) return null;

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.double ||
        editorItem.type == EEditorType.money ||
        editorItem.type == EEditorType.phone) {
      return TextInputType.number;
    }

    return null;
  }

  List<TextInputFormatter> getInputFormatters(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);

    if (editorItem == null) return <TextInputFormatter>[];

    if (editorItem.type == EEditorType.money) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
        PriceInputFormatter(),
      ];
    }

    if (editorItem.type == EEditorType.int ||
        editorItem.type == EEditorType.phone) {
      return <TextInputFormatter>[
        FilteringTextInputFormatter.digitsOnly,
      ];
    }

    return <TextInputFormatter>[];
  }

  String _getEmptyMessageValidator(String title) {
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (title != null
            ? _removeLastStarSymbolAndLowCase(title)
            : Message.getMessage('EditorWidget_thong_tin'));
  }

  String _getInvalidMessageValidator(String title) {
    return Message.getMessage('EditorWidget_vui_long_nhap') +
        ' ' +
        (title != null
            ? _removeLastStarSymbolAndLowCase(title)
            : Message.getMessage('EditorWidget_thong_tin')) +
        ' ' +
        Message.getMessage('EditorWidget_hop_le');
  }

// #endregion TextField

// #region Title
  String _removeLastStarSymbolAndLowCase(String str) {
    if (str != null) {
      return str.replaceAll('*', '').toLowerCase();
    }
    return '';
  }

  String getTitleByFieldName(String fieldName) {
    return _getEditorItemByFieldName(fieldName)?.title;
  }

// #endregion Title

// #region Image
  bool _checkIsImagesKey(String fieldName) {
    final editorItem = _getEditorItemByFieldName(fieldName);
    if (editorItem != null && editorItem.type == EEditorType.image) {
      return true;
    }
    return false;
  }

  Future<void> _saveImages(
      Map<String, List<Map<String, Object>>> response) async {
    final imageEditorItems = editorItems
        .where((element) => element.type == EEditorType.image)
        .toList();

    if (imageEditorItems.isEmpty) return;

    final futures = <Future>[];

    for (final imageEditorItem in imageEditorItems) {
      final List<File> images = dataEntity[imageEditorItem.fieldName];
      final uploadImageInfo = imageEditorItem.uploadImageInfo;
      for (var data in response[uploadImageInfo.responseIdsName]) {
        var idDangTin = Methods.getInt(data, 'ID');
        if (idDangTin != null) {
          for (final image in images) {
            futures.add(
              Methods.uploadFileParentChild(
                image,
                uploadImageInfo.tableName,
                -1,
                uploadImageInfo.fieldName,
                uploadImageInfo.parentTableName,
                idDangTin,
              ).then((value) => print("Result upload image: $value"),
                  onError: (err) {
                print('Error upload image: $err');
              }),
            );
          }
        }
      }
    }

    await Future.wait(futures);
  }

// #endregion Image

// #region Validate Data
  bool validate({int step, bool isShowInvalidDialog = true}) {
    if (!_checkValidData(step)) {
      print('validate');
      if (isShowInvalidDialog == true) {
        _isShowInValidDialog.value = true;
      }
      return false;
    } else {
      return true;
    }
  }

  bool _checkValidData(int step) {
    for (final editorItem in editorItems) {
      if ((step != null && editorItem.step != step) || !editorItem.isRequired) {
        continue;
      }
      if (getValidator(editorItem.fieldName) != null) {
        _messageInvalidDialog = editorItem.title != null
            ? _getEmptyMessageValidator(editorItem.title)
            : null;
        return false;
      }
    }
    return true;
  }

  bool _validateString(dynamic value) {
    return (value is String) && value.isNotEmpty;
  }

  bool _validateMoney(dynamic value) {
    return (value is num && value > 0);
  }

  bool _isEmpty(String fieldName) {
    if (dataEntity[fieldName] == null ||
        (dataEntity[fieldName] is String &&
            (dataEntity[fieldName] as String).isEmpty)) {
      return true;
    }
    return false;
  }

// #endregion Validate Data

  void _resetAfterDone() {
    _isWaiting.value = false;
    _succeed.value = null;
    _isDirtyData = false;
    _addDefaultValues();
  }

  // void _resetAfterFail() {
  //   _isWaiting.value = false;
  //   _succeed.value = false;
  //   _isDirtyData = true;
  // }

  void _addDefaultValues() {
    editorItems.forEach((item) {
      dataEntity.addAll({item.fieldName: item.defaultValue});
    });
  }

  EditorItem _getEditorItemByFieldName(String fieldName) {
    final editorItem = editorItems.firstWhere(
        (element) => element.fieldName == fieldName,
        orElse: () => null);
    return editorItem;
  }

  bool _validateImage(List images) {
    if (images != null && images.isNotEmpty) {
      return true;
    }
    return false;
  }
}
